package org.rcisoft.bus.jnsp7xtmdqsj.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.Stopwatch;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmOptionDto;
import org.rcisoft.bus.jnsp7xtm.vo.Jnsp7xtmVo;
import org.rcisoft.bus.jnsp7xtmdqsj.dao.Jnsp7xtmDqsjRepository;
import org.rcisoft.bus.jnsp7xtmdqsj.dto.Jnsp7xtmDqsjExportTemplateDto;
import org.rcisoft.bus.jnsp7xtmdqsj.dto.Jnsp7xtmDqsjImportDto;
import org.rcisoft.bus.jnsp7xtmdqsj.dto.Jnsp7xtmDqsjPageHandleDto;
import org.rcisoft.bus.jnsp7xtmdqsj.entity.Jnsp7xtmDqsj;
import org.rcisoft.bus.jnsp7xtmdqsj.service.Jnsp7xtmDqsjService;
import org.rcisoft.bus.taskinfo.dao.TaskInfoRepository;
import org.rcisoft.bus.taskinfo.entity.TaskInfo;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.enums.CyImportErrCodeEnum;
import org.rcisoft.util.ExportToExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by cy on 2024年5月14日 上午10:20:48.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class Jnsp7xtmDqsjServiceImpl extends ServiceImpl<Jnsp7xtmDqsjRepository,Jnsp7xtmDqsj>  implements Jnsp7xtmDqsjService {

    // 前四个按钮的锁
    private static Lock topFourButtonLock = new ReentrantLock();

    //
    private static Lock finallyButtonLockLock = new ReentrantLock();

    @Autowired
    private TaskInfoRepository taskInfoRepository;

    @Value("${jnsp7xtmDqsj.batchAddPagesize}")
    private Integer batchInsertSize;

    @Value("${jnsp7xtmDqsj.pageQuestSize}")
    private Integer pageQuerySize;

    @Value("${jnsp7xtmDqsj.shema}")
    private String shema;

    @Value("${jnsp7xtmDqsj.importBatchAddSize}")
    private Integer importBatchAddSize;

    /**
    * 保存
    * @param jnsp7xtmDqsj
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(Jnsp7xtmDqsj jnsp7xtmDqsj){
        //增加操作
        int line = baseMapper.insert(jnsp7xtmDqsj);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            jnsp7xtmDqsj.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除
    * @param jnsp7xtmDqsj
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(Jnsp7xtmDqsj jnsp7xtmDqsj){
        jnsp7xtmDqsj.setDeleted();
        int line = baseMapper.deleteById(jnsp7xtmDqsj);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            jnsp7xtmDqsj.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 修改
    * @param jnsp7xtmDqsj
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(Jnsp7xtmDqsj jnsp7xtmDqsj){
        int line = baseMapper.updateById(jnsp7xtmDqsj);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ jnsp7xtmDqsj.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询
    * @param id
    * @return
    */
    @Override
    public Jnsp7xtmDqsj findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询
    * @param jnsp7xtmDqsj
    * @return
    */
    @Override
    public IPage<Jnsp7xtmDqsj> findAllByPagination(CyPageInfo<Jnsp7xtmDqsj> paginationUtility,
        Jnsp7xtmDqsj jnsp7xtmDqsj){
        return baseMapper.queryJnsp7xtmDqsjsPaged(paginationUtility,jnsp7xtmDqsj);

    }

    /**
    * 查询list
    * @param jnsp7xtmDqsj
    * @return
    */
    @Override
    public List<Jnsp7xtmDqsj> findAll(Jnsp7xtmDqsj jnsp7xtmDqsj){
        return baseMapper.queryJnsp7xtmDqsjs(jnsp7xtmDqsj);

    }

    /**
    * 导出
    * @return
    */
    @Override
    public List<Jnsp7xtmDqsj> export(Jnsp7xtmDqsj jnsp7xtmDqsj) {
        List<Jnsp7xtmDqsj> jnsp7xtmDqsjList = baseMapper.queryJnsp7xtmDqsjs(jnsp7xtmDqsj);
        return jnsp7xtmDqsjList;
    }

    @Override
    public Jnsp7xtmOptionDto getMonth() {
        Jnsp7xtmOptionDto jnsp7xtmOptionDto = new Jnsp7xtmOptionDto();
        List<String> monthList = baseMapper.getMonthList();
        jnsp7xtmOptionDto.setLabel("month");
        jnsp7xtmOptionDto.setPullDownList(monthList);
        return jnsp7xtmOptionDto;
    }

    // 数据表重置
    @Override
    public CyPersistModel dataTableRese() {
        try {
            if (topFourButtonLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    baseMapper.truncateJnsp7xtmDqsj(shema);
                }  catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    topFourButtonLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new CyPersistModel(1);
    }

    /**
     * 月度表拉取
     * **/
    @Override
    @Async
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    public CyPersistModel monthlyTablePull(String taskId) {

        try {
            if (topFourButtonLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    // 创建并启动计时器
                    Stopwatch stopwatch = Stopwatch.createStarted();
                    // 获取数据的总数
                    Integer total = baseMapper.queryTotalJnsp7xtmDqs();
                    Integer pageNumber = 1;
                    if (total >= pageQuerySize) {
                        pageNumber = (int) Math.ceil((double) total / pageQuerySize);
                    }
                    for (int i = 1; i <= pageNumber ; i++) {
                        Jnsp7xtmDqsjPageHandleDto jnsp7xtmDqsjPageHandleDto = Jnsp7xtmDqsjPageHandleDto.builder().page(i).rows(pageQuerySize).build();
                        addJnsp7xtmDqsjByPage(jnsp7xtmDqsjPageHandleDto);
                    }
//                    Thread.sleep(5000);
                    LambdaUpdateWrapper<TaskInfo> updateWrapper = new LambdaUpdateWrapper<>();
                    updateWrapper.eq(TaskInfo::getTaskId,taskId).set(TaskInfo::getStatus,"1");
                    taskInfoRepository.update(null,updateWrapper);
                    log.info("数据同步需要多少秒" + stopwatch.elapsed().getSeconds());
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new CyServiceException(501,"数据同步异常");
                }
                finally {
                    topFourButtonLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new CyPersistModel(1);
    }

    public void addJnsp7xtmDqsjByPage(Jnsp7xtmDqsjPageHandleDto jnsp7xtmDqsjPageHandleDto) {
        log.info("页数: {}",jnsp7xtmDqsjPageHandleDto.getPage());
        List<Jnsp7xtmDqsj> addList = new ArrayList<>();
        baseMapper.streamQuery(resultContext -> {
            Jnsp7xtmVo resultObject = resultContext.getResultObject();
            Jnsp7xtmDqsj jnsp7xtmDqsj = Jnsp7xtmDqsj.generateJnsp7xtmDqsj(resultObject);
            jnsp7xtmDqsj.setFlag("1");
            jnsp7xtmDqsj.setCreateDate(new Date());
            jnsp7xtmDqsj.setUpdateDate(new Date());
            jnsp7xtmDqsj.setDelFlag("0");
            addList.add(jnsp7xtmDqsj);
            if (addList.size() > batchInsertSize) {
                log.info("addList",addList);
                baseMapper.batchAddJnsp7xtmDqsj(addList);
                addList.clear();
            }
        },jnsp7xtmDqsjPageHandleDto);
        if(CollectionUtil.isNotEmpty(addList))
            baseMapper.batchAddJnsp7xtmDqsj(addList);
    }

    /**添加任务**/
    @Override
    public TaskInfo addTaskInfo() {
        TaskInfo taskInfo = new TaskInfo();
        try {
            if (topFourButtonLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    taskInfo.setStatus("0");
                    taskInfo.setTaskId(CyIdGenUtil.uuid());
                    taskInfo.setFlag("1");
                    taskInfoRepository.insert(taskInfo);
                    return taskInfo;
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new CyServiceException(501,"数据同步异常");
                } finally {
                    topFourButtonLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return taskInfo;
    }

    /**
     * 月度表重置
     * **/
    @Override
    public CyPersistModel monthlyReset(Jnsp7xtmDqsjPageHandleDto jnsp7xtmDqsjPageHandleDto) {
        try {
            if (topFourButtonLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    Jnsp7xtmDqsjPageHandleDto requestCondition = this.acquireRequestCondition(jnsp7xtmDqsjPageHandleDto);
                    requestCondition.setType("1");
                    int i = baseMapper.deleteJnsp7xtmDqs(requestCondition);
                }  catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    topFourButtonLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new CyPersistModel(1);
    }

    /**
     * 客户表重置
     ***/
    @Override
    public CyPersistModel clientReset(Jnsp7xtmDqsjPageHandleDto jnsp7xtmDqsjPageHandleDto) {
        try {
            if (topFourButtonLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    Jnsp7xtmDqsjPageHandleDto requestCondition = this.acquireRequestCondition(jnsp7xtmDqsjPageHandleDto);
                    requestCondition.setType("2");
                    int i = baseMapper.deleteJnsp7xtmDqs(requestCondition);
                }  catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    topFourButtonLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new CyPersistModel(1);
    }


    /**
     * 下载空模板
     * **/
    @Override
    public void jnsp7xtmDqsjDownEmptyTemplate(Jnsp7xtmDqsjPageHandleDto jnsp7xtmDqsjPageHandleDto, HttpServletResponse response) {
        // 导出空模板
        OutputStream outputStream = null;
        Workbook workbook = null;
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("试题数据信息.xlsx", "UTF-8"));
            ExportParams exportParams = new ExportParams();
            exportParams.setType(ExcelType.XSSF);
            List<Jnsp7xtmDqsjExportTemplateDto> exportList = new ArrayList<>();
            workbook = ExcelExportUtil.exportExcel(exportParams, Jnsp7xtmDqsjExportTemplateDto.class, exportList);
            Sheet sheet = workbook.getSheetAt(0);
            String[] termList = new String[]{"L"};
            ExportToExcelUtil.XSSFSetDropDownAndHidden(termList,workbook,sheet,1, 100000, 0, 0);
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
        } catch (IOException e) {
            throw new CyServiceException(e.getMessage());
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException var11) {
                log.info("context:" + var11);
            }
        }

    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    public Jnsp7xtmDqsjImportDto jnsp7xtmDqsjImportDtoExcel(MultipartFile file) {
        Jnsp7xtmDqsjImportDto jnsp7xtmDqsjImportDto = new Jnsp7xtmDqsjImportDto();
        try {
            if (finallyButtonLockLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    // 要添加的数据行数据
                    int rows = 0;
                    // 要添加的list
                    List<Jnsp7xtmDqsj> jnsp7xtmDqsjList = new ArrayList<>();
                    List<Jnsp7xtmDqsjExportTemplateDto> dataList = ExportToExcelUtil.importExcelNotTitle(file, 1, 1, Jnsp7xtmDqsjExportTemplateDto.class);
                    if (CollectionUtil.isEmpty(dataList)) {
                        throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"导入模板为空");
                    }
                    for (int i = 0; i < dataList.size(); i++) {
                        /**
                         * 1.校验 month和cc和pn 必填
                         * **/
                        if (StringUtils.isNotBlank(dataList.get(i).getMonth()) && StringUtils.isNotBlank(dataList.get(i).getCc()) && StringUtils.isNotBlank(dataList.get(i).getPn())) {
                            /**
                             * 2.校验 amt,rate,up,qty,fc,po 格式是数据
                             * **/
                            if ((dataList.get(i).getMonth().length() != 6 && this.isValidDate(dataList.get(i).getMonth())))
                                 throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行month格式不对");

                             if (!StringUtils.isEmpty(dataList.get(i).getAmt())) {
                                 if (!isValidIntegerOrLessThanNDecimal(dataList.get(i).getAmt(),7))
                                     throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行amt格式不对");
                             }
                             if (!StringUtils.isEmpty(dataList.get(i).getRate())) {
                                 if (!isValidIntegerOrLessThanNDecimal(dataList.get(i).getRate(),6))
                                     throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行rate格式不对");
                             }
                            if (!StringUtils.isEmpty(dataList.get(i).getUp())) {
                                if (!isValidIntegerOrLessThanNDecimal(dataList.get(i).getUp(),5))
                                    throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行rate格式不对");
                            }
                            if (!StringUtils.isEmpty(dataList.get(i).getQty())) {
                                if (!isValidIntegerOrLessThanNDecimal(dataList.get(i).getQty(),2))
                                    throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行qty格式不对");
                            }
                            if (!StringUtils.isEmpty(dataList.get(i).getFc())) {
                                if (!isValidIntegerOrLessThanNDecimal(dataList.get(i).getFc(),2))
                                    throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行fc格式不对");
                            }
                            if (!StringUtils.isEmpty(dataList.get(i).getPo())) {
                                if (!isValidIntegerOrLessThanNDecimal(dataList.get(i).getPo(),2))
                                    throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),"第"+(i+1)+"行po格式不对");
                            }
                            /**
                             * 3.转换jnsp7xtmDqsj
                             * **/
                            Jnsp7xtmDqsj jnsp7xtmDqsj = new Jnsp7xtmDqsj();
                            if (StringUtils.isNotEmpty(dataList.get(i).getCc())) {
                                List<Jnsp7xtmDqsj> dqsjList = baseMapper.queryCustomer(dataList.get(i).getCc());
                                if (CollectionUtil.isNotEmpty(dqsjList)) {
                                    jnsp7xtmDqsj = dqsjList.get(0);
                                }
                            }
                            Jnsp7xtmDqsj finallyJnsp7xtmDqsj = Jnsp7xtmDqsj.generateJnsp7xtmDqsj(dataList.get(i), jnsp7xtmDqsj);
                            if (StringUtils.isNotEmpty(dataList.get(i).getPn())) {
                                List<String> list = baseMapper.querySypnByPn(dataList.get(i).getPn());
                                if (CollectionUtil.isNotEmpty(list)) {
                                    finallyJnsp7xtmDqsj.setSypn(list.get(0));
                                }
                            }
                            jnsp7xtmDqsjList.add(finallyJnsp7xtmDqsj);
                            rows = rows + 1;
                            if (jnsp7xtmDqsjList.size() == importBatchAddSize) {
                              this.importBatchAddList(jnsp7xtmDqsjList);
                                jnsp7xtmDqsjList.clear();
                            }
                        }
                    }
                    if (CollectionUtil.isNotEmpty(jnsp7xtmDqsjList)) {
                        baseMapper.batchAddJnsp7xtmDqsj(jnsp7xtmDqsjList);
                    }

                    jnsp7xtmDqsjImportDto.setTotal(dataList.size());
                    jnsp7xtmDqsjImportDto.setRows(rows);
                    return jnsp7xtmDqsjImportDto;
                }  catch (Exception e) {
                    e.printStackTrace();
                    throw new CyServiceException(CyImportErrCodeEnum.IMPORT_ERR.getCode(),e.getMessage());
                } finally {
                    finallyButtonLockLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return jnsp7xtmDqsjImportDto;
    }

    private Jnsp7xtmDqsjPageHandleDto acquireRequestCondition(Jnsp7xtmDqsjPageHandleDto jnsp7xtmDqsjPageHandleDto) {
        Jnsp7xtmDqsjPageHandleDto requestJnsp7xtmDqs = new Jnsp7xtmDqsjPageHandleDto();
        BeanUtil.copyProperties(jnsp7xtmDqsjPageHandleDto,requestJnsp7xtmDqs);
        return requestJnsp7xtmDqs;
    }

    private void importBatchAddList(List<Jnsp7xtmDqsj> list) {
        baseMapper.batchAddJnsp7xtmDqsj(list);
    }

    /**校验月份的格式是否正确**/
    public boolean isValidDate(String str) {
        String pattern = "^\\d{4}\\d{2}$"; // 前四位是年，后两位是月份
        Pattern r = Pattern.compile(pattern);
        Matcher m = r.matcher(str);
        return m.matches();
    }

    public static boolean isValidIntegerOrLessThanNDecimal(String str, int n) {
        try {
            BigDecimal bd = new BigDecimal(str);
            if (bd.scale() <= n) {
                return true;
            }
        } catch (NumberFormatException e) {
        }
        return false;
    }

}
